/*
*            Copyright  2004 Rosiello Security
*
*		   http://www.rosiello.org
*
**********************************************************************
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 1, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
***********************************************************************
*
* AUTHOR : Angelo Rosiello
* CONTACT: angelo@rosiello.org
*
*/

#include "library.h"
#include "check_proxy.h"

#define MAX_SOCKETS 150
#define TIMEOUT 3

#define S_NONE       0
#define S_CONNECTING 1
#define IP_LOG "ip.log"

int status;

struct conn_t {
  int s;
  char status;
  time_t a;
  struct sockaddr_in addr;
};
struct conn_t connlist[MAX_SOCKETS];

void init_sockets();
void check_sockets();
void log_host();
void fatal(char *);

//MAIN
int main(int argc, char *argv[])
{
  int done, i, cip, bb, ret, k, ns, fd;
  time_t scantime;
  char ip[20];
  status = 0;

  fprintf( stdout, "\nCopyright  2004 Rosiello Security\n" )	;
  fprintf( stdout, "http://www.rosiello.org\n\n" )		;
  fprintf( stdout, "ProxyScanner v2.0 by Angelo Rosiello\n\n")	;


  if (argc < 3)
  {
     printf("\nUsage: %s <b-block> <port> [c-block]\n", argv[0]);
     return -1;
  }

  if( strlen(argv[1]) > 19 )
  {
    printf("No way\n");
    exit( 0 );
  }


  done = 0; cip = 1; bb = 0;

  if ( argc >= 4 )
  {
     bb = atoi( argv[3] );
     if ( (bb < 0) || (bb > 255) )
        fatal( "Invalid b-range.\n" );
  }


  fd = open( IP_LOG, O_RDONLY );
  if( fd >= 0 )
  {
	  remove( IP_LOG )	;
	  close( fd )		;
  }

  init_sockets( );

  scantime = time( 0 );

  while( !done ) {
    for (i = 0; i < MAX_SOCKETS; i++) {
        if ( cip == 255 ) {
           if ( (bb == 255) || (argc >= 4) ) {
              ns = 0;
              for (k = 0; k < MAX_SOCKETS; k++) {
                  if (connlist[k].status > S_NONE) {
                     ns++;
                     break;
                  }
              }

              if ( ns == 0 )
                 done = 1;

              break;
           }
           else {
             cip = 0;
             bb++;
           }
        }

        if ( connlist[i].status == S_NONE ) {
           connlist[i].s = socket( AF_INET, SOCK_STREAM, 0 );
           if ( connlist[i].s == -1 )
              printf( "Unable to allocate socket.\n" );
           else {
             ret = fcntl( connlist[i].s, F_SETFL, O_NONBLOCK );
             if (ret == -1) {
                printf("Unable to set O_NONBLOCK\n");
                close( connlist[i].s );
             }
             else {
               memset( (char *)ip, 0, 20 );
               sprintf( ip, "%s.%d.%d", argv[1], bb, cip  );
               connlist[i].addr.sin_addr.s_addr = inet_addr(ip);
               if (connlist[i].addr.sin_addr.s_addr == -1)
                 fatal( "Invalid IP." );
               connlist[i].addr.sin_family = AF_INET;
               connlist[i].addr.sin_port = htons(atoi(argv[2]));
               connlist[i].a = time(0);
               connlist[i].status = S_CONNECTING;
               cip++;
             }
           }
        }
    }

    check_sockets( );
  }

  printf( "\nScan completed in %u seconds.\n\n", (time(0) - scantime) );
  if( status==1 )
  {
  	printf( "Starting the validation phase\n" );
	printf( "Check the file proxy_ok.log when finished\n" );
  	check_proxy( argv[2] );
  }
  else printf( "No proxy found!\n" );
}


//REQUIRES: true
//EFFECTS : initializes sockets
void init_sockets( void )
{
  int i;

  for (i = 0; i < MAX_SOCKETS; i++)
  {
      connlist[i].status = S_NONE;
      memset((struct sockaddr_in *)&connlist[i].addr, 0,
             sizeof(struct sockaddr_in));
  }
}


void check_sockets( )
{
  int i, ret;

  for (i = 0; i < MAX_SOCKETS; i++) {
      if ((connlist[i].a < (time(0) - TIMEOUT)) &&
          (connlist[i].status == S_CONNECTING)) {
         close(connlist[i].s);
         connlist[i].status = S_NONE;
      }

      else if (connlist[i].status == S_CONNECTING) {
         ret = connect(connlist[i].s,
                       (struct sockaddr *)&connlist[i].addr,
                       sizeof(struct sockaddr_in));
         if (ret == -1) {
            if (errno == EISCONN) {
               printf("%15s - %2u second(s)\n",
                      (char *)inet_ntoa(connlist[i].addr.sin_addr),
                      (time(0)-connlist[i].a));
               log_host((char *)inet_ntoa(connlist[i].addr.sin_addr));
	       close(connlist[i].s);
               connlist[i].status = S_NONE;
            }

            if ((errno != EALREADY) && (errno != EINPROGRESS)) {
               close(connlist[i].s);
               connlist[i].status = S_NONE;
            }
         }
         else {
           printf("%15s - %2u second(s)\n",
                  (char *)inet_ntoa(connlist[i].addr.sin_addr),
                  (time(0)-connlist[i].a));
	log_host((char *)inet_ntoa(connlist[i].addr.sin_addr));
           close(connlist[i].s);
           connlist[i].status = S_NONE;
         }
      }
  }
}

//OVERVIEW: error management
void fatal( char *err )
{
  int i;

  printf("Error: %s\n", err);
  for (i = 0; i < MAX_SOCKETS; i++) {
      if (connlist[i].status >= S_CONNECTING)
         close(connlist[i].s);
  }
  exit( -1 );
}


//REQUIRES: h!=NULL
//EFFECTS : Log the host in the file "ip.log"
void log_host( char *h )
{

	char buffer[1024];
	int fd;

	if(strlen(h) > 1023)
	{
		printf("HostName too long!\n");
		exit( -1 );
	}
	sprintf(buffer, "%s\n", h);
	fd = open( IP_LOG, O_CREAT | O_WRONLY | O_APPEND, 0644 );
	write( fd, buffer, strlen(buffer) );
	close( fd );
	status=1;
}
